﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace Filter
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            this.grid.InitializeLayout += new InitializeLayoutEventHandler(grid_InitializeLayout);
            this.grid.DataSource = this.Table;

            //  Add a custom mapping that shows the dropdown when the
            //  forward slash key is pressed
            GridKeyActionMapping mapping =
                new GridKeyActionMapping(
                    (Keys)191,
                    UltraGridAction.ToggleDropdown,
                    0,
                    UltraGridState.InEdit,
                    SpecialKeys.All,
                    0);

            this.grid.KeyActionMappings.Add(mapping);

            //  Eat the KeyPress for the '/' char since we are using
            //  it to show the dropdown
            KeyPressEventHandler keyPress =
            delegate(object sender, KeyPressEventArgs e)
            {
                if ( e.KeyChar == '/' )
                    e.Handled = true;
            };
            this.grid.KeyPress += keyPress;
        }

        void grid_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            UltraGridLayout layout = e.Layout;

            UltraGridColumn col = layout.Bands[0].Columns["Manager"];

            UltraDropDown dd = new UltraDropDown();
            dd.DataSource = this.Table2;
            col.ValueList = dd;

            col.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownValidate;
        }

        private DataTable Table
        {
            get
            {
                DataTable table = new DataTable();
                table.Columns.Add("ID", typeof(string));
                table.Columns.Add("Name", typeof(string));
                table.Columns.Add("Manager", typeof(string));

                table.Rows.Add( new object[]{ "1", "One", DBNull.Value } );
                table.Rows.Add( new object[]{ "2", "Two", DBNull.Value } );
                table.Rows.Add( new object[]{ "3", "Three", DBNull.Value } );
                table.Rows.Add( new object[]{ "4", "Four", DBNull.Value } );
                table.Rows.Add( new object[]{ "5", "Five", DBNull.Value } );


                return table;
            }
        }

        private DataTable Table2
        {
            get
            {
                DataTable table = new DataTable();
                table.Columns.Add("Name", typeof(string));

                table.Rows.Add( new object[]{ "Moe" } );
                table.Rows.Add( new object[]{ "Larry" } );
                table.Rows.Add( new object[]{ "Curly" } );
                table.Rows.Add( new object[]{ "Shemp" } );
                table.Rows.Add( new object[]{ "Joe" } );

                return table;
            }
        }

        #region ZeroFilter class
        private class ZeroFilter : FilterCondition
        {
            private const string DisplayName = "Almost Zero";
            private int precision = 2;
            private string colName = null;

            public ZeroFilter(string colName, int precision = 2)
            {
                if ( string.IsNullOrEmpty(colName) )
                    throw new ArgumentException("colName");

                if ( precision < 0 )
                    throw new ArgumentException("precision");

                this.precision = precision;
                this.colName = colName;
            }

            public override bool MeetsCriteria(UltraGridRow row)
            {
                //  Get the value of the cell in the colName column
                object cellValue = row != null && row.Cells.Exists(this.colName) ? row.Cells[colName].Value : null;

                //  Round the value to the number of decimal places specified
                //  by the precision property and return a value that indicates
                //  whether that value is effectively zero after the rounding.
                if ( cellValue is decimal )
                {
                    decimal value = (decimal)cellValue;
                    value = Math.Round(value, this.precision);
                    return value == 0m;
                }

                return base.MeetsCriteria(row);
            }

            public override string ToString()
            {
                return ZeroFilter.DisplayName;
            }
        }
        #endregion ZeroFilter class
    }
}
